@@ -2,6 +2,8 @@
 #include "Matrix.h"
 
 const string path2abundance = "/assemblies/metag/ContigStats/Coverage.pergene";
+const string path2counts = "/assemblies/metag/ContigStats/Coverage.count_pergene";
+const string pseudoAssMarker = "/assemblies/metag/longReads.fasta.filt.sto";
 
 //typedef std::unordered_map<uint, uint>::iterator SmplAbunIT;
 //typedef std::unordered_map<uint, uint> SmplAbun;
@@ -25,9 +27,10 @@ private:
 class GeneAbundance
 {
 public:
-	GeneAbundance(const string);
+	GeneAbundance(const string, const string);
 	inline smat_fl getAbundance(const string);
 private:
+	bool isPsAss;
 	SmplAbun GeneAbu;
 	//vector<int> ContigID;
 	
@@ -37,10 +40,10 @@ class ClStr2Mat
 {
 	//class for gene catalog creation with cd-hit
 public:
-	ClStr2Mat(const string inF, const string outF, const string mapF, const string baseP);
+	ClStr2Mat(const string inF, const string outF, const string mapF, const string baseP, bool covCalc);
 	virtual ~ClStr2Mat();
 private:
-	void read_map(const string);
+	void read_map(const string,bool);
 	void printVec(ofstream&, vector<smat_fl>&,const string&);
 
 	vector<GeneAbundance*> GAs;
@@ -49,6 +52,6 @@ private:
 	vector<string> smplLoc;
 	vector<string> baseP;
 	size_t smplN;
-	int curr ;
+	size_t curr ;
 };
 
